# -*- coding: utf-8 -*-

# Vendor Library
from sqlalchemy import or_

# Project Library
from kinder.core.declaration_status.models import DSS

from gisdo.algorithm import BaseApplicationAlgorithm
from gisdo.alchemy_models import (
    AlchemyWorkType, AlchemyDeclarationStatus)


class ApplicationCountByAge(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """
        Количество заявлений по возрастам
        """

        return self._get_base_query(**kwargs)


class ApplicationCountByDelivery(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """
        По типу подачи: с портала или при личном обращении
        """

        portal = kwargs.get('portal', 0)
        base = self._get_base_query()

        return self.session.query(
            base).filter(
                base.c.portal == portal).subquery()


class ApplicationCountByGroupType(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """
        Группы кратковременного пребывания
        """

        base = self._get_base_query(**kwargs)
        return self.session.query(
            base).join(
                AlchemyWorkType).filter(
                    AlchemyWorkType.code == '3-5').subquery()


class ApplicationCountByDenied(BaseApplicationAlgorithm):

    def get_result_set(self, **kwargs):
        """
        Количество отказов.
        """

        base = self._get_base_query(**kwargs)
        return (
            self.session.query(
                base).join(
                    AlchemyDeclarationStatus)
            .filter(or_(AlchemyDeclarationStatus.code == DSS.DIDNT_COME,
                        AlchemyDeclarationStatus.code == DSS.REFUSED))
            .subquery())
